import uuid

from decimal import Decimal

from pydantic import BaseModel, Field, validator

from . import to_camel
from .customer import CustomerLink
from .modifier import ModifierLink
from .product_link import ProductLink
from .table import TableLink
from .tax import TaxLink


class Inventory(BaseModel):
    id_: uuid.UUID | None
    product: ProductLink
    quantity: Decimal = Field(multiple_of=0.01)
    price: Decimal | None
    tax: TaxLink | None
    tax_rate: Decimal | None
    discount: Decimal = Field(ge=0, multiple_of=0.00001, le=1)
    is_happy_hour: bool
    modifiers: list[ModifierLink] | None
    amount: Decimal | None

    class Config:
        alias_generator = to_camel

    @validator("amount", always=True)
    def calculate_amount(cls, v, values):
        is_happy_hour = values["is_happy_hour"] if "is_happy_hour" in values else False
        price = values["price"] if "price" in values and not is_happy_hour else 0
        quantity = values["quantity"] if "quantity" in values else 0
        discount = values["discount"] if "discount" in values else 0
        tax_rate = values["tax_rate"] if "tax_rate" in values else 0
        return round(Decimal(price * quantity * (1 - discount) * (1 + tax_rate)), 2)


class Kot(BaseModel):
    id_: uuid.UUID | None
    inventories: list[Inventory]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class VoucherIn(BaseModel):
    pax: int
    table: TableLink
    customer: CustomerLink | None
    kots: list[Kot]

    class Config:
        fields = {"id_": "id"}
        anystr_strip_whitespace = True
        alias_generator = to_camel


class Voucher(VoucherIn):
    id_: uuid.UUID

    class Config:
        fields = {"id_": "id"}
        anystr_strip_whitespace = True
        alias_generator = to_camel
