import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

import { HeaderFooter } from './header-footer';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/header-footer';
const serviceName = 'HeaderFooterService';

@Injectable({
  providedIn: 'root',
})
export class HeaderFooterService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  list(): Observable<HeaderFooter[]> {
    return this.http
      .get<HeaderFooter[]>(url)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<HeaderFooter[]>;
  }

  save(item: HeaderFooter): Observable<HeaderFooter[]> {
    return this.http
      .post<HeaderFooter[]>(url, item, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<HeaderFooter[]>;
  }
}
