import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { BehaviorSubject } from 'rxjs';
import { Observable } from 'rxjs';
import { debounceTime, distinctUntilChanged, startWith } from 'rxjs/operators';

import { MenuCategory } from '../../core/menu-category';
import { Product } from '../../core/product';
import { SaleCategory } from '../../core/sale-category';

import { TemporalProductListDatasource } from './temporal-product-list-datasource';

@Component({
  selector: 'app-product-list',
  templateUrl: './temporal-product-list.component.html',
  styleUrls: ['./temporal-product-list.component.css'],
})
export class TemporalProductListComponent implements OnInit {
  searchFilter: Observable<string> = new Observable();
  menuCategoryFilter: BehaviorSubject<string> = new BehaviorSubject('');
  saleCategoryFilter: BehaviorSubject<string> = new BehaviorSubject('');
  data: BehaviorSubject<Product[][]> = new BehaviorSubject<Product[][]>([]);
  dataSource: TemporalProductListDatasource = new TemporalProductListDatasource(
    this.searchFilter,
    this.menuCategoryFilter,
    this.saleCategoryFilter,
    this.data,
  );

  form: FormGroup;
  list: Product[][] = [];
  menuCategories: MenuCategory[] = [];
  saleCategories: SaleCategory[] = [];
  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns: string[] = [
    'name',
    'price',
    'menuCategory',
    'saleCategory',
    'info',
    'quantity',
  ];

  constructor(private route: ActivatedRoute, private fb: FormBuilder) {
    this.form = this.fb.group({
      filter: '',
      menuCategory: '',
      saleCategory: '',
    });
    this.data.subscribe((data: Product[][]) => {
      this.list = data;
    });
    this.searchFilter = (this.form.get('filter') as FormControl).valueChanges.pipe(
      startWith(''),
      debounceTime(150),
      distinctUntilChanged(),
    );
  }

  filterMcOn(val: string) {
    this.menuCategoryFilter.next(val);
  }

  filterScOn(val: string) {
    this.saleCategoryFilter.next(val);
  }

  ngOnInit() {
    this.dataSource = new TemporalProductListDatasource(
      this.searchFilter,
      this.menuCategoryFilter,
      this.saleCategoryFilter,
      this.data,
    );
    this.route.data.subscribe((value) => {
      const data = value as {
        list: Product[][];
        menuCategories: MenuCategory[];
        saleCategories: SaleCategory[];
      };
      this.loadData(data.list, data.menuCategories, data.saleCategories);
    });
  }

  loadData(list: Product[][], menuCategories: MenuCategory[], saleCategories: SaleCategory[]) {
    this.menuCategories = menuCategories;
    this.saleCategories = saleCategories;
    this.data.next(list);
  }
}
